@@ -563,7 +563,7 @@ extractRowsPBM(const struct pam * const inpamP,
     int             row;
     unsigned int    totalWidth;
 
-    assert(0 <= leftcol && leftcol <= rightcol && rightcol < inpamP->width);
+    assert(leftcol <= rightcol);
     assert(toprow <= bottomrow);
 
     if (leftcol > 0) {
